@@ -17,7 +17,8 @@ import numpy as n, sys, _deconv
 # Find smallest representable # > 0 for setting clip level
 lo_clip_lev = n.finfo(n.float).tiny 
 
-def clean(im, ker, mdl=None, gain=.1, maxiter=10000, tol=1e-3, verbose=False):
+def clean(im, ker, mdl=None, gain=.1, maxiter=10000, tol=1e-3, 
+        stop_if_div=True, verbose=False):
     """This standard Hoegbom clean deconvolution algorithm operates on the 
     assumption that the image is composed of point sources.  This makes it a 
     poor choice for images with distributed flux.  In each iteration, a point 
@@ -43,7 +44,8 @@ def clean(im, ker, mdl=None, gain=.1, maxiter=10000, tol=1e-3, verbose=False):
                                    n.fft.fft2(ker)).astype(im.dtype)
         else: raise ValueError('Number of dimensions != 1 or 2')
     iter = _deconv.clean(res, ker, mdl,
-            gain=gain, maxiter=maxiter, tol=tol, verbose=int(verbose))
+            gain=gain, maxiter=maxiter, tol=tol, 
+            stop_if_div=int(stop_if_div), verbose=int(verbose))
     score = n.sqrt(n.average(n.abs(res)**2))
     info = {'success':iter > 0 and iter < maxiter, 'tol':tol}
     if iter < 0: info.update({'term':'divergence', 'iter':-iter})
