@@ -8,6 +8,34 @@ import numpy as n, utils, coord, pyfits, time
 deg2rad = n.pi / 180.
 rad2deg = 180. / n.pi
 
+def word_wrap(string, width=80,ind1=0,ind2=0,prefix=''):
+    """ word wrapping function.
+        string: the string to wrap
+        width: the column number to wrap at
+        prefix: prefix each line with this string (goes before any indentation)
+        ind1: number of characters to indent the first line
+        ind2: number of characters to indent the rest of the lines
+    """
+    awidth = min(width-2-len(prefix+ind1*' '),width-2-len(prefix+ind2*' '))
+    words = string.split(' ')
+    okwords = []
+    chunk = lambda v, l: [v[i*l:(i+1)*l] for i in range(int(n.ceil(len(v)/float(l))))]
+    for word in words:
+        for okword in chunk(word,awidth):
+            okwords.append(okword)
+    lines = []
+    l = prefix+ind1*' '
+    for i,w in enumerate(okwords):
+        #print w,len(l+' '+w),width
+        if len(l+' ' + w)<width:
+            l += ' '+w
+        else:
+            lines.append(l)
+            l = prefix + ind2*' '+w
+    lines.append(l)
+    return '\n'.join(lines)+'\n'
+    
+    
 def recenter(a, c):
     """Slide the (0,0) point of matrix a to a new location tuple c.  This is
     useful for making an image centered on your screen after performing an
@@ -29,7 +57,7 @@ def convolve2d(a, b):
 def gaussian_beam(sigma, shape=0, amp=1., center=(0,0)):
     """Return a 2D gaussian.  Normalized to area under curve = 'amp'.  
     Down by 1/e at distance 'sigma' from 'center'."""
-    if type(shape) == type(0): shape = array([2, 2]) * sigma
+    if type(shape) == type(0): shape = n.array([2, 2]) * sigma
     def gaussian(x, y):
         nx = n.where(x > shape[0] / 2, x - shape[0], x)
         ny = n.where(y > shape[1] / 2, y - shape[1], y)
@@ -39,7 +67,8 @@ def gaussian_beam(sigma, shape=0, amp=1., center=(0,0)):
     return recenter(g, center)
 
 def beam_gain(bm):
-    return n.sqrt((n.abs(bm)**2).sum())
+    #return n.sqrt((n.abs(bm)**2).sum())
+    return n.abs(bm).max()
 
 class Img:
     """Class for gridding uv data, recording the synthesized beam profile,
@@ -65,6 +94,17 @@ class Img:
         mask = n.where(L**2 + M**2 >= 1, 1, 0)
         L,M = n.ma.array(L, mask=mask), n.ma.array(M, mask=mask)
         return recenter(L, center), recenter(M, center)
+    def get_indices(self, u, v):
+        """Get the pixel indices corresponding to the provided uv coordinates."""
+        u = n.round(u / self.res).astype(n.int)
+        v = n.round(v / self.res).astype(n.int)
+        return n.array([-v,u],).transpose()
+    def get_uv(self):
+        """Return the u,v indices of the pixels in the uv matrix."""
+        u,v = n.indices(self.shape)
+        u = n.where(u < self.shape[0]/2, u, u - self.shape[0])
+        v = n.where(v < self.shape[1]/2, v, v - self.shape[1])
+        return u*self.res, v*self.res
     def put(self, (u,v,w), data, wgts=None, apply=True):
         """Grid uv data (w is ignored) onto a UV plane.  Data should already
         have the phase due to w removed.  Assumes the Hermitian conjugate
@@ -83,9 +123,7 @@ class Img:
         else:
             uv = n.zeros_like(self.uv)
             bm = [n.zeros_like(i) for i in self.bm]
-        u = n.round(u / self.res).astype(n.int)
-        v = n.round(v / self.res).astype(n.int)
-        inds = n.array([-v,u],).transpose()
+        inds = self.get_indices(u,v)
         ok = n.logical_and(n.abs(inds[:,0]) < self.shape[0],
             n.abs(inds[:,1]) < self.shape[1])
         data = data.compress(ok)
@@ -260,7 +298,7 @@ def to_fits(filename, data, clobber=False,
         object='', telescope='', instrument='', observer='', origin='AIPY',
         obs_date=time.strftime('%D'), cur_date=time.strftime('%D'), 
         ra=0, dec=0, d_ra=0, d_dec=0, epoch=2000., 
-        freq=0, d_freq=0, bscale=0, bzero=0):
+        freq=0, d_freq=0, bscale=0, bzero=0,history=''):
     """Write image data to a FITS file.  Follows convention of VLA image
     headers.  "axes" describes dimensions of "data" provided.  (ra,dec) are
     the degree coordinates of image center in the specified "epoch". 
@@ -296,10 +334,20 @@ def to_fits(filename, data, clobber=False,
         elif ax.lower().startswith('stokes'): val,delta = (1, 1)
         else: val,delta = (0,0)
         phdu.header.update('CTYPE%d' % (i+1), ax.upper())
+        if ax.lower().startswith('ra') or ax.lower().startswith('dec'):
+            phdu.header.update('CRPIX%d' % (i+1), 
+                    round(phdu.data.shape[-(i+1)]/2.))
+        else:
+            phdu.header.update('CRPIX%d' % (i+1), phdu.data.shape[-(i+1)])
         phdu.header.update('CRVAL%d' % (i+1), val)
         phdu.header.update('CDELT%d' % (i+1), delta)
         phdu.header.update('CROTA%d' % (i+1), 0)
-        phdu.header.update('CRPIX%d' % (i+1), phdu.data.shape[-(i+1)])
+    if history!='':
+        history = history.split("\n")
+        for line in history:
+            if len(line)>1:
+                for subline in word_wrap(line,70,5,10,'#').split("\n"):
+                    phdu.header.add_history(subline)
     phdu.header.update('ORIGIN', origin)
     phdu.header.update('DATE', cur_date, comment='FILE WRITTEN ON DD/MM/YY')
     pyfits.writeto(filename, phdu.data, phdu.header, clobber=True)
@@ -333,3 +381,75 @@ def from_fits(filename):
         except(KeyError): pass
     kwds['axes'] = axes
     return data, kwds
+    
+def find_axis(phdu,name):
+    #find the axis number for RA
+    for k in phdu.header.items():
+        if k[0].lower().startswith('ctype'):
+            if k[1].lower().startswith(name):            
+                return int(k[1][5])
+
+
+def from_fits_to_fits(infile,outfile,data,kwds,history=None):
+    """
+    Create a fits file in outfile with data using header from infile using
+    kwds to override values in the header.
+    See img.to_fits for typical header variables.
+    """
+
+    phdu = pyfits.open(infile)[0]
+    axes = []
+    for i in range(1,phdu.header.get('naxis')):
+        type = "CTYPE"+str(i)
+        axes.append(phdu.header.get(type))
+    print axes
+    data.shape = data.shape + (1,) * (len(axes) - len(data.shape))
+    phdu.data = data.transpose()
+    for i,ax in enumerate(axes):
+        if ax.lower().startswith('ra'):
+             if kwds.has_key('ra'): val=kwds['ra']
+             else: val=None
+             if kwds.has_key('d_ra'):delta = kwds['d_ra']
+             else: delta=None
+        elif ax.lower().startswith('dec'):
+             if kwds.has_key('dec'): val=kwds['dec']
+             else: val=None
+             if kwds.has_key('d_dec'):delta = kwds['d_dec']
+             else: delta=None
+        elif ax.lower().startswith('freq'):
+             if kwds.has_key('freq'): val=kwds['freq']
+             else: val=None
+             if kwds.has_key('d_freq'):delta = kwds['d_freq']
+             else: delta=None
+        else: val,delta = None,None
+#        elif ax.lower().startswith('dec') and kwds.has_key: val,delta = (dec, d_dec)
+#        elif ax.lower().startswith('freq'): val,delta = (freq, d_freq)
+#        elif ax.lower().startswith('stokes'): val,delta = (1, 1)
+#        else: val,delta = (0,0)
+        phdu.header.update('CTYPE%d' % (i+1), ax.upper())
+        if ax.lower().startswith('ra') or ax.lower().startswith('dec'):
+            phdu.header.update('CRPIX%d' % (i+1), 
+                    round(phdu.data.shape[-(i+1)]/2.))
+        else:
+            phdu.header.update('CRPIX%d' % (i+1), phdu.data.shape[-(i+1)])
+        if not val is None: phdu.header.update('CRVAL%d' % (i+1), val);
+        if not delta is None: phdu.header.update('CDELT%d' % (i+1), delta)
+        phdu.header.update('CROTA%d' % (i+1), 0)
+    for k,v in kwds.iteritems():
+        try:
+            phdu.header.update(k,v)
+            print "updated %s with %s"%(k,str(v))
+        except(ValueError): 
+            print "error on %s "%(k,)
+            continue
+    if history is None:history = "from_fits_to_fits: from %s to %s"%(infile,
+                outfile)
+    history = history.split("\n")
+    for line in history:
+        if len(line)>1:
+            for subline in word_wrap(line,70,5,10,'#').split("\n"):
+                phdu.header.add_history(subline)
+    print phdu.header
+    pyfits.writeto(outfile, phdu.data, phdu.header, clobber=True)
+
+    
